/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * CandleStickPanel.java
 *
 * Created on 16 January 2002, 22:15
 */

package com.exponto.jstock;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.Transparency;
import javax.swing.JPanel;
import org.apache.log4j.Logger;

import com.exponto.jstock.indicators.Indicator;
import com.exponto.util.*;
//import com.exponto.util.FifoQueue;
import java.util.ArrayList;
import java.text.DecimalFormat;

/**
 * Handles all drawing in the candlestick panel including candlesticks,
 * moving average, axis, and lines.
 * 
 * Have some key and mouse event handling I would like to move to another
 * class one day. 
 * 
 * @author  kaare
 * @version 
 */

public class CandleStickPanel extends JPanel {
	static Logger logger = Logger.getLogger(CandleStickPanel.class);
	private static Dimension currentCSPDim;
    double x1 = 0.0;
    double x2 = 0.0;
    double y1 = 0.0;
    double y2 = 0.0;
    // positive number indicates which stick is selected, -1 if none
	int stickNumber = -1;
	// If "newLine" shall be printed or not
	boolean printLine = false;
	boolean newLine = true;
    int width, height = 0;
    int numberOfSticksToDisplay;
//    int maDays = 0; // Number of days that the current moving avarage list is calculted over
    
//    boolean shiftPressed = false;
//	boolean altPressed = false;
//	boolean ctrlPressed = false;
	
    int tMargin;
    int bMargin;
    int lMargin;
    int rMargin;    
    int pixPrDay;
    
    double scaleY = 0.0;
    double logMin, logMax = 0.0; 
    LogDataSeries logDS;    
    private static UserPreferences userPrefs = UserPreferences.getInstance();
    Lines drawnLines;
    
    // Used to draw indicators (and save as bufferedImage)
	private GraphicsParameters graphicsParameters = new GraphicsParameters();
	private GraphicsParameters oldGraphicsParameters; 
	private ArrayList imageList = new ArrayList();
	private BufferedImage bImage = null;  
    /**
     * Creates new CandleStickPanel using the data in
     * lDS to draw the graphs 
     *
     */
    
    public CandleStickPanel(LogDataSeries lDS)  {
    	logDS = lDS;
		drawnLines = new Lines(logDS.getTickerId() , logDS.isWeekly());
		if (userPrefs.getEnableDB()) {
			JstockDB jDB = JstockDB.getInstance();
			drawnLines = jDB.loadLines(drawnLines);
		}
        initGraphicVariables();
        setVisible(true);
		repaint(); 
    }
    /**
     * Sets the size available for this window.
     * 
     */
	public static void setAvailableSize(Dimension dim) {
		// TODO: There may be some timing issues as it seems  
		// that the size once in a while is used before it is set
		currentCSPDim = dim;
	}

    public void keyPressed(int key) {
//		if (key == KeyEvent.VK_SHIFT) {
//			shiftPressed = true;
//		} else if (key == KeyEvent.VK_ALT) {
//			altPressed = true;
//		} else if (key == KeyEvent.VK_CONTROL) {
//			ctrlPressed = true;
//		}
    	if (userPrefs.getDrawMode()) {
    		// In x direction movment is limited to about the candlesticksize due
    		// to that the date is used for persitence
    		int movePixel = 1;
			if (key == KeyEvent.VK_RIGHT) {
				moveLine(userPrefs.getPixelPrStick(), 0);
				repaint();
			} else if (key == KeyEvent.VK_LEFT) {
				moveLine(-userPrefs.getPixelPrStick(), 0);
				repaint();
			} else if (key == KeyEvent.VK_UP) {
				moveLine(0, -movePixel);
				repaint();
			} else if (key == KeyEvent.VK_DOWN) {
				moveLine(0, movePixel);
				repaint();
			}
    	} else {
    		if (key == KeyEvent.VK_RIGHT) {
				stickNumber++;
				repaint();
    		} else if (key == KeyEvent.VK_LEFT) {
				stickNumber--;
				repaint();
    		}
    	}
    }
    
//	public void keyReleased(int key) {
//		if (key == KeyEvent.VK_SHIFT) {
//			shiftPressed = false;
//		} else if (key == KeyEvent.VK_ALT) {
//			altPressed = false;
//		} else if (key == KeyEvent.VK_CONTROL) {
//			ctrlPressed = false;
//		}
//	}
	
	public void keyTyped(char ch) {
		if (userPrefs.getDrawMode()) {
			// \u007f is delete key
			if (ch == '\u007f' || ch ==  'd') {
				drawnLines.deleteActiveSelection();
				repaint();
			} else if (ch ==  'n') {
				//selectNextLine
				if (drawnLines.selectNext(false)){
					repaint();	
				}
			} else if (ch ==  'N') {
				//selectNextEnd of line
				drawnLines.selectNextEnd();
				repaint();	
			} else if (ch ==  'p') {
				// select Previous line
				if (drawnLines.selectPrevious(false)){
					repaint();	
				}
			} else if (ch ==  '\u0013') {
				
				drawnLines.saveLines();
				logger.debug("Saving lines for " + logDS.getTicker());
				//System.out.println(" Ctrl s ");
				repaint();
			}
		} else {
			if (ch == 'n') {
				//Next
				stickNumber++;
				repaint();
			} else if (ch == 'p') {
				// Previous
				stickNumber--;
				repaint();
			}
		}
	}

    public void mouseDrag(int x, int y) {
		Line2D line = new Line2D.Double();
		if(newLine) {
			logger.error("Should not execute this code, newLine is: " + newLine);
			//System.out.println(" Why here? newLine is: " + newLine);
			newLine = false;
			printLine = false;
			x1 = x;
			y1 = y;
		} else {
			printLine = true;
			x2 = x;
			y2 = y;
			setVisible(true);
			this.repaint();               
		}
    }
    public void mouseClick(int x, int y) {
		//Request focus for keyboard input
		boolean focus = this.requestFocusInWindow(false);
//		System.out.println(" Focus: " + focus);
//		Draw line mode
    	if (!userPrefs.getDrawMode()) {
    		newLine = true;
//			Find current stick and select it
			stickNumber = (x - lMargin) /pixPrDay;
			this.repaint(); 
			//System.out.println(" Selected stick is number: " + stickNumber);
    	}
//		System.out.println(" X: " + x+ ", Y: " + y);	
    }
	public void mousePress(int x, int y) {
		// Draw line mode
		if (userPrefs.getDrawMode()) {
			stickNumber = -1;
			if(newLine) {
				newLine = false;
				printLine = false;
				x1 = x;
				y1 = y;
			} else {
				newLine = true;
				printLine = true;
				x2 = x;
				y2 = y;                
			}
			this.repaint();
		} 
	}
	
	public void mouseRelease(int x, int y) {
		// Draw line mode
		if (userPrefs.getDrawMode()) {
			stickNumber = -1;
			if(newLine) {
				newLine = false;
				printLine = false;
				x1 = x;
				y1 = y;
			} else {
				newLine = true;
				printLine = true;
				x2 = x;
				y2 = y;
				// TODO: Problems with lines (x-axis) beyond latest date
				// Partly fixed the initial draw part, but need to take into account that only working
				// days are drawn. Also need to fix the part that (re)draws the persistent lines. 
//				System.out.println(" Drawn line: " + x1 + " " + y1 + ", " + x2 + " " + y2);
				// Doesn't draw lines if neither x & y dimension is wider than 5 pixels
				if (Math.abs(x1-x2) > 5 || Math.abs(y1-y2) > 5) { 
					drawnLines.addLine(this.convertPoints(x1,y1,x2,y2));
					drawnLines.saveLines();
				}
				this.repaint();              
			}
		}
	}	
	/**
	 * 
	 * @param y	coordinate (within this panel) 
	 * @return a value for the y coordinate
	 */
	public double getValue(double y) {
		double valueY = Math.exp(((height + tMargin -y)/ scaleY + logMin) * Constants.log10);
		return valueY;
	}
	
	public Date getDate(double x){
		Date dateX = null;
		int dayNumber = (int) Math.round((x-lMargin)/pixPrDay) + logDS.getStartOfPeriode();
		if (dayNumber < logDS.logStock.size()) {
			LogTradingData lTD = logDS.getResult(dayNumber);
			dateX = new Date(lTD.day.year, lTD.day.month, lTD.day.day);
		} else {
			logger.info("Date is outside the dataseries");
		}
		return dateX;
	}
	
    public void paint(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        super.paint(g2d);
        this.setPreferredSize(currentCSPDim);
        this.setMaximumSize(currentCSPDim);
		graphicsParameters.setDimension(currentCSPDim);
		graphicsParameters.setTopMargin(tMargin);
		graphicsParameters.setBottomMargin(bMargin);
        // Constants must be added otherwise the display size is changed
		width = (int) currentCSPDim.getWidth() - lMargin - rMargin;
		height = (int) currentCSPDim.getHeight() - tMargin - bMargin;
		numberOfSticksToDisplay = (int) (width / userPrefs.getPixelPrStick());
//		logger.debug("CandleStickPanel, width, height: " + currentCSPDim.getWidth() + ", " + currentCSPDim.getHeight());
//		logger.debug("CandleStickPanel, real width, height: " + this.getWidth() + ", " + this.getHeight());
//		System.out.println("Class ChartPanel, Calculated number of sticks: " + numberOfSticksToDisplay);
// 		Will display as many sticks as window size permits from latest day
		logDS.setCurrentPeriode(numberOfSticksToDisplay);
        //drawXaxis(g2d);
		logger.debug(" CandleStickPanel LogMin, LogMax: "+ logMin + ", " + logMax);
        if (logDS.isWeekly()) {
			drawWeeklyNumbers(g2d);
//			logger.debug("CSP paint for " + logDS.getTicker() + " size is " + logDS.logStock.size());        	
        } else {
			drawWeekNumbers(g2d);
        }
        drawStatusField(g2d);
		resetGraphicsVariables();
        drawAxis(g2d);
        drawChandelSticks(g2d);
		// Candlestickpanel indicators
		drawIndicators(g2d);
		drawLines(g2d);
    }
	
    private void drawChandelSticks(final Graphics2D g2d){
        int w, y0 = 34;
        double h, y, x, x1, y1, y2 = 0.0;
        double scaleVol = 0.0;
        double volPix = height * userPrefs.getScalingFactorChartVolume();
        scaleVol = volPix/(logDS.maxVol);
		//System.out.println("maks vol er:" + logDS.maxVol + " height: " + height);
        y0 = height + tMargin;
        LogTradingData lTD = new LogTradingData();
        Line2D line = new Line2D.Double ();
        Rectangle2D rect = new Rectangle2D.Double ();
        logDS.resetCurrentPeriode();
        int i = 0;
      	while (logDS.hasNextLTD()) {
			lTD = logDS.getNextLTD();
			if (lTD == null) {
				logger.error("lTD is null, should not happend");  
// 			System.out.println("CandleStickPanel, drawCandleSticks: Error - lTD is null");	
			} else if (!lTD.isUsable()) {
				if (lTD.status == Constants.faulty || lTD.status == Constants.drop) {
					g2d.setColor(Color.black);
					x = i * pixPrDay + lMargin;
					g2d.drawString( String.valueOf(lTD.status) , (float) x, tMargin + 6);
				}
      		} else {
      			// This is the normal
				y = y0 - scaleY*(lTD.logX-logMin);
				h = scaleY*lTD.logDiff;
				x = i * pixPrDay + lMargin;
				rect.setRect(x, y, pixPrDay -1, h);
				x1 = x + pixPrDay/2 - 1;
				y1 = y0 - scaleY*(lTD.logHigh-logMin);
				y2 = y0 - scaleY*(lTD.logLow-logMin);
//				draws volume  
				g2d.setColor(Color.red);
				line.setLine(x1,y0, x1, (y0 - (lTD.volume * scaleVol)));
				g2d.draw(line);
				// Draws line between hih and low            
				g2d.setColor(Color.black);            
				line.setLine(x1, y1, x1, y2);
				g2d.draw(line);
//				Draws rectangle
				if (h == 0.0) {
					line.setLine(x, y, x + pixPrDay -1, y);
					g2d.draw(line);
				} else if (lTD.open < lTD.close) g2d.setColor(Color.white);
				g2d.fill(rect);
      		}
			i++;	
      	}
    }
        
    private void drawAxis(final Graphics2D g2d) {
        // trinn er int br vre double
        int x, yStart, yStop = 0;
        double minorSupportLineLevel = 60;
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(2);
		df.setDecimalSeparatorAlwaysShown(false);
		g2d.setColor(Color.black);
        x = lMargin;
        yStart = tMargin;
        yStop = yStart + height;
        //Draws X-axis
        Line2D line = new Line2D.Double (x, yStop, x + width, yStop);
        g2d.draw(line);
        //Draws Y-axis
        line = new Line2D.Double(x, yStop, x, yStart) ;
        g2d.draw(line);
		g2d.setColor(Color.DARK_GRAY);
		double previousY = yStop;
        // Draw logaritmic axis from min to max
        for (double i =  logDS.yMin; i <= logDS.yMax; i = i + logDS.stepSize) {
            double y = height + tMargin - scaleY*((Math.log(i)/Constants.log10)-logMin);
            line.setLine(width + lMargin, y, lMargin-5, y);
            g2d.draw(line);
            g2d.drawString(df.format(i), 10, (int) y+5);
            if ((previousY - y) > (double) userPrefs.getMinorLineLevelDays()) {
            	// Draws the minor support levels
				previousY = y;
            	double temp = y;
            	double tempI = i - logDS.stepSize/2;
            	y = height + tMargin - scaleY*((Math.log(tempI)/Constants.log10)-logMin);
				line.setLine(width + lMargin, y, lMargin-5, y);
				g2d.setColor(Color.GRAY);
				g2d.draw(line);
				//str = String.valueOf(tempI );
 				//if (str.length() > 5) str = str.substring(0,5);
				g2d.drawString(df.format(tempI), 10, (int) y+5);
				g2d.setColor(Color.black);
				
            } else {
				previousY = y;
            }
            
            if (i < logDS.stepSize) {
            	// This fixes the problem with odd values on y axis due to large difference 
            	// between high and low. See also findYScaleLimits() in LogDataSeries
            	// First value will be odd, next ok
            	i = 0;
            }
        }
		g2d.setColor(Color.black);
    }     
    
    private void drawWeekNumbers(final Graphics2D g2d) {
		Line2D line = new Line2D.Double ();
//		Due to the little oddity that getNextWeekNumber also takes care of finding 
//		new hi's and low's in the periode
		logDS.resetCurrentPeriode();
		int numberOfPix = lMargin + logDS.numberOfDaysInFirstWeek() * pixPrDay;
// The first week, no line is currently drawn for that
		int currentWeek = logDS.getNextWeekNumber();
//		int currentMonth;
		int pixPrWeek = 5 * pixPrDay;
		int y0 = height + tMargin;
		g2d.setColor(Color.black);
		//System.out.println("Frste hele uke: "+ currentWeek);
		currentWeek = logDS.getNextWeekNumber();
		while (currentWeek != 0) {
			// Draws line before "current week"
			line.setLine(numberOfPix, y0, numberOfPix, y0+5);
			g2d.draw(line);
			numberOfPix = numberOfPix + pixPrWeek;
			g2d.drawString(new Integer (currentWeek).toString(), (numberOfPix - (4 * pixPrDay)),(int) y0+20);
			currentWeek = logDS.getNextWeekNumber();
		}
    }

	private void drawWeeklyNumbers(final Graphics2D g2d) {
//		TODO: Bug - Align axis correctly. Seems that java & system calender does not agree on week numbers from 2000 and earlier 2001 and up seems ok.
		Line2D line = new Line2D.Double ();
//		Due to the little oddity that getNextWeekNumber also takes care of finding 
//		new hi's and low's in the periode
		logDS.resetCurrentPeriode();
		// Start drawing at 3.rd stick
		int numberOfPix = lMargin;
		int currentWeek;
		int y0 = height + tMargin;
		int i = 1;
		g2d.setColor(Color.black);
		currentWeek = logDS.getNextWeekNumber();
		//System.out.println("Frste hele uke: "+ currentWeek);
		while (currentWeek != 0) {
			numberOfPix = numberOfPix + pixPrDay;
			// draw every 5. th weeknumber
			if (i == 5) {
				line.setLine(numberOfPix + 1, y0, numberOfPix + 1, y0+i);
				g2d.draw(line);
				g2d.drawString(new Integer (currentWeek).toString(), (numberOfPix - 4),(int) y0+20);
				i = 0;
			}
			i ++;
			currentWeek = logDS.getNextWeekNumber();
		}
	}
	private void drawLines(final Graphics2D g2d) {
		// See comment in Lines class (addLine)
		// Also need to be able to delete lines
		// Print lines if in draw mode and line is "complete"
		if (userPrefs.getDrawMode()) {
			drawnLines.reset();
			PersistentLine pL = drawnLines.hasNext();
			//Line2D line = new Line2D.Double ();
			Line2D line;
			int selectedEnd;
			Rectangle2D rect = new Rectangle2D.Double();
//			System.out.println(" Retrieved lines: ");
			while (pL != null ) {
				line = this.convertPersistentLine(pL);    
//				pL.setSelected(true);
				if(!pL.isDeleted()) {
					if (pL.isSelected()) {
						selectedEnd = pL.getSelected();
						//Line2D border = new Line2D.Double();
						int delta = 3;
						if (selectedEnd == 2) {
//							Other end selected
							g2d.setColor(Color.DARK_GRAY);
						} else {
							g2d.setColor(Color.yellow);
						}
						double x = line.getX1();
						double y = line.getY1();
						rect.setRect(x-delta, y-delta, 2*delta, 2*delta);
						g2d.fill(rect);
						if (selectedEnd == 1) {
							// Other end selected
							g2d.setColor(Color.DARK_GRAY);
						} else {
							g2d.setColor(Color.yellow);
						}
						x = line.getX2();
						y = line.getY2();
						rect.setRect(x-delta, y-delta, 2*delta, 2*delta);
						g2d.fill(rect);
						g2d.setColor(Color.black);
					}
					g2d.setColor(Color.black);
					g2d.draw(line);
				}
				pL = drawnLines.hasNext();
//				System.out.println(" Line: " + line.getX1() + " " + line.getY1() + ", " + line.getX2() + " " + line.getY2());
			}
			
			// For temporary printing while "dragging" line
			if (printLine) {
				g2d.setColor(Color.black);
				Line2D tempLine = new Line2D.Double();
				tempLine.setLine(x1, y1, x2, y2);    
				g2d.draw(tempLine);
			}
		}
	}
	
	private void drawIndicators(final Graphics2D g2d) {
		IndicatorController indicatorController = IndicatorController.getInstance();
		ArrayList indicatorList = indicatorController.getEnabledCandleStickIndicators();
		if (imageList.size() != indicatorList.size()) {
			
			// Initialising the imageList with null in order to avoid any problems with get/set and 0pointers  
			imageList.clear();
			for (int i = 0; i <indicatorList.size();i++) {
				Indicator indicator = (Indicator) indicatorList.get(i);
				indicator.initiateIndicator(logDS);
				imageList.add(null);
			}
			logger.debug("ImageList is reset");
		}
		int y = height + tMargin + 35;
		int x = width + lMargin;
		int length;
		Line2D line = new Line2D.Double();
		String indicatorString;
		BufferedImage bufferedImage;
		for (int i = 0; i <indicatorList.size();i++) {
			bImage = (BufferedImage) imageList.get(i);
			Indicator indicator = (Indicator) indicatorList.get(i);
			if (!this.imageValid(indicator)) {
				
				int width =  (int) graphicsParameters.getDimension().getWidth();
				int height = (int) graphicsParameters.getDimension().getHeight();
				bImage = g2d.getDeviceConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
				// Get the Graphics
				Graphics2D bImageGraphics = bImage.createGraphics();
				//bImageGraphics.setBackground(Color.lightGray);
				indicator.paintIndicator(bImageGraphics, graphicsParameters);

//				bImage = this.drawImage(indicator);
				imageList.set(i, bImage);
				logger.debug("Indicator not valid");
			}
			g2d.drawImage(bImage, 0, 0, this);
			//indicator.initiateIndicator(logDS);
			
			//indicator.paintIndicator(g2d, graphicsParameters);
			
			
			g2d.setColor(indicator.getColor());			
			line.setLine(x - 25 , y -5, x , y - 5);
			g2d.draw(line);
			indicatorString = indicator.toString();
			length = indicatorString.length() * 8;
			x = x - length - 25;
			g2d.setColor(Color.black);
			g2d.drawString(indicatorString, x , y);
			x = x -10;
		}
	}
	
	private void drawStatusField(final Graphics2D g2d) {
		double high, low;
		String selected = "";
		String indicators = "";
		Date first;
		Date last;
		LogTradingData lTD;
		lTD = logDS.getResult(logDS.getStartOfPeriode());
		first = lTD.day; 
		lTD = logDS.getResult(logDS.getEndOfPeriode());
		last = lTD.day;
		high = logDS.tradedHighInPeriode;
		low = logDS.tradedLowInPeriode;
		if (!userPrefs.getDrawMode()) {
			int startPeriode = logDS.getStartOfPeriode();
			int lengthPeriode = logDS.getEndOfPeriode() - startPeriode;
			if (stickNumber >= 0 &&  stickNumber <= lengthPeriode) {
				lTD = logDS.getResult(startPeriode + stickNumber);
				selected = "  SELECTED - Day: " + lTD.day.toString() + " Open: " + lTD.open +  " High: " + lTD.high + " Low: "  + lTD.low + " Close: " + lTD.close + " Vol: " + lTD.volume;
				markSelectedStick(g2d, lTD);
				indicators = this.getIndicatorText(stickNumber);
			} else {
				selected = "";
			}
		}
		g2d.drawString("PERIODE - From: " + first.toString() + " To: " + last.toString() + " High: " + new Double (high).toString() + " Low: " + new Double (low).toString() + selected, lMargin, tMargin-6);
		g2d.drawString(indicators, lMargin, height + tMargin + 35);
	}
	private void markSelectedStick(final Graphics2D g2d, LogTradingData lTD){
		if (stickNumber >= 0) {
			Line2D line = new Line2D.Double ();
			int x =lMargin + pixPrDay * stickNumber -1;
			double y0 =	height + tMargin;
			double y1 = y0 - scaleY*(lTD.logHigh-logMin);
			// orange may give better visibility for the candlestick itself
			g2d.setColor(Color.gray);
			double y;
			boolean draw = true;
			// Draw dashed line
			for (y = y1; y < y0; y = y + 4) {
				if (draw) {
					line.setLine(x, y, x, y+4);    
					g2d.draw(line);
					line.setLine(x + pixPrDay, y, x + pixPrDay, y + 4);
					g2d.draw(line);
 				}
				draw = !draw;
			}
			// Use this for solid line
			line.setLine(x, y1, x + pixPrDay, y1);
			g2d.draw(line);
			g2d.setColor(Color.black); 
		
		}
	}
	
	private String getIndicatorText(int dayNumber) {
		IndicatorController indicatorController = IndicatorController.getInstance();
		ArrayList indicatorList = indicatorController.getEnabledCandleStickIndicators();
		String indicatorPrintOut = "";
		for (int i = 0; i <indicatorList.size();i++) {
			Indicator indicator = (Indicator) indicatorList.get(i);
			indicatorPrintOut = indicatorPrintOut + indicator.getDailyPrintOut(dayNumber) + "   ";
		}
		return indicatorPrintOut;
	}

	/**
	 * Converts the (pixel) start and end points of a line to a persistent line that
	 * can be stored a database and be redrawn later.
	 * 
	 * @param x1 A double representing the x1 coordinate
	 * @param y1 A double representing the y1 coordinate
	 * @param x2 A double representing the x2 coordinate
	 * @param y2 A double representing the y2 coordinate
	 * @return a PersistentLine representing the input parameters
	 */
	private PersistentLine convertPoints(double x1, double y1, double x2, double y2) {
		Date dateX2;
		double valueY1 = Math.exp(((height + tMargin -y1)/ scaleY + logMin) * Constants.log10);
		double valueY2 = Math.exp(((height + tMargin -y2)/ scaleY + logMin) * Constants.log10);
		int dayNumber = (int) Math.round((x1-lMargin)/pixPrDay) + logDS.getStartOfPeriode();
		LogTradingData lTD = logDS.getResult(dayNumber);
		Date dateX1 = new Date(lTD.day.year, lTD.day.month, lTD.day.day);
		dayNumber = (int) Math.round((x2-lMargin)/pixPrDay) + logDS.getStartOfPeriode();
		if (dayNumber >= logDS.logStock.size()) {
			int field;
			if(logDS.isWeekly()) {
				field = java.util.Calendar.WEEK_OF_YEAR;
			} else {
				field = java.util.Calendar.DAY_OF_MONTH;
			}
			lTD = logDS.getResult(logDS.logStock.size()-1);
			dateX2 = Date.getNewDate(lTD.day, field, dayNumber - logDS.logStock.size());
		
		} else {
			lTD = logDS.getResult(dayNumber);
			dateX2 = new Date(lTD.day.year, lTD.day.month, lTD.day.day);
		}
		
		PersistentLine pL = new PersistentLine(dateX1, valueY1, dateX2, valueY2);
//		System.out.println("x1: " + dateX1.toString() +" y1: " + valueY1 + " x2: " + dateX2.toString() + " y2: " + valueY2);
		return pL;
	}

	/**
	 * Converts a point given with with pixel values to a PeristentPoint
	 * with a date and a value.
	 * 
	 * @param x A double representing the x coordinate
	 * @param y A double representing the y coordinate
	 * @return a PersistentPoint representing the input parameters
	 */
	private PersistentPoint convertPoint(double x, double y) {
		Date dateX = null;
		double valueY = Math.exp(((height + tMargin -y)/ scaleY + logMin) * Constants.log10);
		int dayNumber = (int) Math.round((x-lMargin)/pixPrDay) + logDS.getStartOfPeriode();
		if (dayNumber < logDS.logStock.size()) {
			LogTradingData lTD = logDS.getResult(dayNumber);
			dateX = new Date(lTD.day.year, lTD.day.month, lTD.day.day);
		} else {
			logger.info("Date is outside the dataseries");
		}
		return new PersistentPoint(dateX, valueY);
	}
	
	
	private Line2D convertPersistentLine(PersistentLine pL) {
		double y1 =  (height + tMargin) - scaleY * (Math.log(pL.getY1())/Constants.log10 - logMin);
		double y2 =  (height + tMargin) - scaleY * (Math.log(pL.getY2())/Constants.log10 - logMin);
		int x = logDS.findNumberInPeriode(pL.getX1());
		double x1 = x * pixPrDay + lMargin;
		x = logDS.findNumberInPeriode(pL.getX2());
		double x2 = x * pixPrDay + lMargin;
		Line2D l2D = new Line2D.Double(x1, y1, x2, y2);
		return l2D;
	}
	
	private void moveLine(int x, int y) {
		// Should be moved out of here at some time
		PersistentLine pL = drawnLines.getActive();
		if (pL.isSelected()) {
			int selected = pL.getSelected();
			Line2D line = this.convertPersistentLine(pL);
			double x1 = line.getX1();
			double y1 = line.getY1();
			double x2 = line.getX2();
			double y2 = line.getY2();
			System.out.println("Moving x, y: " + x + ", " + y);
			if (selected == 0) {
				x1 = x1 + x;
				y1 = y1 + y;
				x2 = x2 + x;
				y2 = y2 + y;
			} else if (selected == 1) {
				x1 = x1 + x;
				y1 = y1 + y;
			} else if (selected == 2) {
				x2 = x2 + x;
				y2 = y2 + y;
			}
			pL = convertPoints(x1, y1, x2, y2);
			drawnLines.setActive(pL);
		}
	}

	private boolean imageValid(Indicator ind) {
		boolean isValid = true;
		// Initialise oldGrahicsParameters on first run (will always be equal)
		if (oldGraphicsParameters == null) {
			oldGraphicsParameters = (GraphicsParameters) graphicsParameters.clone();
			isValid = false;
			logger.debug("Image is not valid due to: No oldGraphics" );
		} 
		if (bImage == null) {
			isValid = false;
			logger.debug("Image is not valid due to: bImage is null" );
		}
		// Check if indicator parameters has changed
		if (ind.hasChanged()) {
			isValid = false;
			ind.initiateIndicator(logDS);
			logger.debug("Image is not valid due to: Indicator has changed" );
		}
		if (!graphicsParameters.isEqual(oldGraphicsParameters)) {
			isValid = false;
			logger.debug("Image is not valid due to: Graphics has changed" );
		} 
		//logger.debug("Image is " + isValid);
		return isValid;
	}
	
//	private BufferedImage drawImage(Indicator indicator){
//		// Get the Graphics
//		Graphics2D bImageGraphics = bImage.createGraphics();
//		//bImageGraphics.setBackground(Color.lightGray);
//		indicator.paintIndicator(bImageGraphics, graphicsParameters);
//		logger.debug("BufferedImage painted");
//		return bImage;
//	}

	
	private void resetGraphicsVariables() {
		// Resetting this may cause serious harm, if the currentPeriode in logDS
		// isn't what you expected to be
		logDS.findYScaleLimits(height);
		logMin = Math.log(logDS.yMin)/Constants.log10;
		logMax = Math.log(logDS.yMax)/Constants.log10; 
		graphicsParameters.setLogaritmicMaximumValue(logMax);
		graphicsParameters.setLogaritmicMinimumValue(logMin);
		scaleY = height/(logMax - logMin);
	}
    private void initGraphicVariables() {
// 		Initilize a few variables used
        tMargin = userPrefs.getTMargin();
        bMargin = userPrefs.getBMargin();
        lMargin = userPrefs.getLMargin();
        rMargin = userPrefs.getRMargin();
        pixPrDay = userPrefs.getPixelPrStick();
    }
}        

