/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 11-Mar-2004
 *
 */
package com.exponto.jstock;
import java.awt.Dimension;
import org.apache.log4j.Logger;
/**
 * This class encapsulates a few parameters that are needed 
 * in order to draw the indicator correctly on screen. 
 * 
 * @author kaare
 *
 */
public class GraphicsParameters {
	UserPreferences uP;
	private Dimension dim;

	private double logaritmicMinimumValue;
	private double logaritmicMaximumValue;
	
	private int topMargin; // No need to use this for separate panels
	private int bottomMargin; // No need to use this for separate panels
	private int leftMargin;
	private int rightMargin;
	private int pixelPrDay;
	
	static Logger logger = Logger.getLogger(GraphicsParameters.class);
	
	public GraphicsParameters() {
		
	}

	public Object clone() {
		GraphicsParameters clone = new GraphicsParameters();
		clone.setDimension((Dimension) this.getDimension().clone());
		clone.setLogaritmicMaximumValue(this.getLogaritmicMaximumValue());
		clone.setLogaritmicMinimumValue(this.getLogaritmicMinimumValue());
		clone.setTopMargin(this.getTopMargin());
		clone.setBottomMargin(this.getBottomMargin());
		clone.setState();
		return clone;
	}
	
	public boolean isEqual(GraphicsParameters oldGP) {
		boolean equal = true;
		if (this.stateChanged(oldGP)){
			equal = false;
//			logger.debug("GraphicsParameters state changed");
		}
		if (this.getDimension().getHeight() != oldGP.getDimension().getHeight()) {
			equal = false;
//			logger.debug("GraphicsParameters height changed");
		}
		if (this.getDimension().getWidth() != oldGP.getDimension().getWidth()) {
			equal = false;
//			logger.debug("GraphicsParameters width changed");
		}
		if (this.getLogaritmicMaximumValue() != oldGP.getLogaritmicMaximumValue()){
			equal = false;
//			logger.debug("GraphicsParameters logMax changed");
		}
		if (this.getLogaritmicMinimumValue() != oldGP.getLogaritmicMinimumValue()){
			equal = false;
//			logger.debug("GraphicsParameters logMin changed");
		}		
		return equal;
	}
	/**
	 *  Use default settings pr. indicator for indicators with separate panel
	 * @return
	 */
	public int getBottomMargin() {
//		uP =UserPreferences.getInstance();
//		return uP.getBMargin();
		return bottomMargin;
	}

	/**
	 * @return
	 */
	public Dimension getDimension() {
		return dim;
	}

	/**
	 * @return
	 */
	public int getLeftMargin() {
		uP =UserPreferences.getInstance();
		return uP.getLMargin();
	}

	/**
	 * @return
	 */
	public int getPixelPrDay() {
		uP =UserPreferences.getInstance();
		return uP.getPixelPrStick();
	}

	/**
	 * @return
	 */
	public int getRightMargin() {
		uP =UserPreferences.getInstance();
		return uP.getRMargin();
	}

	/**
	 * Use default settings pr. indicator for indicators with separate panel
	 * @return
	 */
	public int getTopMargin() {
//		uP =UserPreferences.getInstance();
//		return uP.getTMargin();
		return topMargin;
	}

	/**
	 * @param dimension
	 */
	public void setDimension(Dimension dimension) {
		dim = dimension;
	}

	public int getNetWidth() {
		return (int) this.getDimension().getWidth() - this.getLeftMargin() - this.getRightMargin();
	}



	/**
	 * @return
	 */
	public double getLogaritmicMaximumValue() {
		return logaritmicMaximumValue;
	}

	/**
	 * @return
	 */
	public double getLogaritmicMinimumValue() {
		return logaritmicMinimumValue;
	}

	/**
	 * @param d
	 */
	public void setLogaritmicMaximumValue(double d) {
		logaritmicMaximumValue = d;
	}

	/**
	 * @param d
	 */
	public void setLogaritmicMinimumValue(double d) {
		logaritmicMinimumValue = d;
	}
	
	private void setState(){
		topMargin = this.getTopMargin();
		bottomMargin = this.getBottomMargin();
		leftMargin = this.getLeftMargin();
		rightMargin = this.getRightMargin();
		pixelPrDay = this.getPixelPrDay();
	}
	
	private boolean stateChanged(GraphicsParameters oldGP) {
		boolean changed = false;
		if (this.getTopMargin() != oldGP.getTopMargin()) {
			changed = true;
		}
		if (this.getBottomMargin() != oldGP.getBottomMargin()){
			changed = true;
		}
		if (this.getLeftMargin() != oldGP.getLeftMargin()){
			changed = true;
		}
		if (this.getRightMargin() != oldGP.getRightMargin()){
			changed = true;
		}
		if (this.getPixelPrDay() != oldGP.getPixelPrDay()){
			changed = true;
		}
		return changed;
	}

	/**
	 * @param bottomMargin The bottomMargin to set.
	 */
	public void setBottomMargin(int bottomMargin) {
		this.bottomMargin = bottomMargin;
	}
	/**
	 * @param topMargin The topMargin to set.
	 */
	public void setTopMargin(int topMargin) {
		this.topMargin = topMargin;
	}
}
