/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Serie.java
 *
 * Created on 16 November 2001, 20:02
 */
package com.exponto.jstock;

/**
 * Super class for storing series of data. Used to implement methods 
 * that was usable for more than one of its subclasses, but only one 
 * subclass remains. Let's say its still here for historic reasons.
 * 
 * @author  kaare
 * @version 0.5
 */

import org.apache.log4j.Logger;
public class Series {
	static Logger logger = Logger.getLogger(Series.class);
    double tradedLowInPeriode = Constants.lowStart;
    double tradedHighInPeriode = 0;
    
    double stepSize = 0.0;    
    final static double log10 = Math.log(10);
    /** Creates new Serie */
    public Series() {
    }
    

    
	/**
	 * Returns a double that are the base (what is the proper name???)
	 * value of dbl. The function is:
	 *          power10(round(log10(dbl)))
	 *
	 * @param dbl   Input double
	 * @return      Base value of double, e.g. (3.1467) returns 1, (0.314)
	 *              returns 0.1.  
	 */    
    public static double base (double dbl) {
        double base = (Math.log(dbl)/log10);
        // Ved konvertering tilbake finner vi riktig "10'er" verdi
        base = Math.floor(base);
        base = Math.pow(10, base);
        return  base;
    }
    
    /**
     * Returns a double that are the n most significant numbers of dbl.
     * 
     * @param dbl   Input double
     * @param n     number of significant numbers to return
     * @return      The n most signifcant numbers of dbl. 
                    The number are always rounded down, so it is always 
                    lower or equal to dbl. E.g (3.1467, 3) will return
                    3.14 and (31467, 2) returns 31000. Internally the
                    dbl is converted to long and the method may cause
                    overflow with large numbers. 
     */    
    public static double findSignificant(double dbl, int n) {

        long value = 0L;
        // Ved konvertering til long blir tallet avrundet til nrmeste heltall
        long base = (long) (Math.log(dbl)/log10);
        // Ved konvertering tilbake finner vi riktig "10'er" verdi
        //base = (long)  Math.pow(10, base);
        // Korrigerer for tall > 1
        if (dbl >= 1) n--;
        double presisjon = Math.pow(10, (n - base));
        double newDbl = dbl * presisjon;
        value = (long) (newDbl);
        return (double) value / presisjon;
    }
    /**
     * Calculates the distance between supportlines used in drawing graphs.
     * This method should be improved. See comments in source. 
     *
     * @param min   lowest value found in the dataseries 
     * @param max  highest value found in the dataseries
	 * @param heigth The height of the graph area 
     * @return A double that is the distance between supportlines that are 
     * drawn i GraphFile.
     */    
    public static double stepSize(double min, double max, int height) {
        // This parameter is used in the calculation of the number of support line
        // To add more lines increase it (it's influence is indirectly).
        int supportLineFactor = 40;
//		Change theese to adjust scaling, should be more flexible
        //int small = 1;
        int medium = 2;
        int large = 3;
        int xLarge = 6;
        int xxLarge = 12;
        int xxxLarge = 18;
        // 13 is ok for heigth of approx 550 pixels.
        int supportLine = height/supportLineFactor;
        double stepSize;
        double scale = (max-min)/supportLine;
        double baseScale = base(scale);
        scale = scale/baseScale;
        //System.out.println(" Scale: "+ scale);
        if (scale > xxxLarge) { stepSize = baseScale * 50;
		} else if (scale > xxLarge) { stepSize = baseScale * 20;	
		} else if (scale > xLarge) { stepSize = baseScale * 10;	
        } else if (scale > large) { stepSize = baseScale * 5;
        } else if (scale > medium) { stepSize = baseScale * 2;
        } else stepSize = baseScale;
        //if (ymin < )
        return stepSize;
    }
    /**
     * Calculates the distance between supportlines used in drawing graphs.
     * This method should be improved. See comments in source. 
     *
     * @param value     Input value
     * @param stepSize  The stepSize
     * @return          A double that is the closest double value equal to 
                        n * value where n is an integer. The result is 
                        always lower than value. */    
    
    public static double roundToStepSize(double value, double stepSize) {
        int n = (int) (value/stepSize);
        return n * stepSize;
    }
    
/*    public static void main(final String[] args) {
        Serie t = new Serie();
        double i = 1234.0;
        i = t.stepSize(16, 33);
        System.out.println("i: " + i);
        i = t.stepSize (2.6, 3.4);
        System.out.println("i: " + i);
        i = t.stepSize (70, 195);
        System.out.println("i: " + i);
        i = t.stepSize(2.5, 7);
        System.out.println("i: " + i);
        i = t.stepSize(4.7, 18);
        System.out.println("i: " + i);
        i = t.stepSize (29, 46);
        System.out.println("i: " + i);
        i = findSignificant(87.67, 2);
        System.out.println("i: " + i);
        i = findSignificant(144.5, 2);
        System.out.println("i: " + i);
        i = findSignificant(3.1467, 2);
        System.out.println("i: " + i);
        i = findSignificant(0.0031467, 4);
        System.out.println("i: " + i);
        i = findSignificant(0.676, 2);
        System.out.println("i: " + i);
        i = findSignificant(987345123, 3);
        System.out.println("i: " + i); 
    } */
}

