/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 29-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.util.ArrayList;
import java.text.DecimalFormat;

import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.GraphicsParameters;
import com.exponto.util.FifoQueue;
import com.exponto.jstock.Constants;
import com.exponto.jstock.LogTradingData;
/**
 * @author kaare
 *
 */
public class MA extends Indicator {

	private static MAParameters maParameters = null;
	protected ArrayList indicatorSeries;
	GraphicsParameters graphicsParameters;
	
	int widthOfDay;
	int numberOfDaysToDisplay;
	double scaleY = 0.01;
	double logMin;
	double logMax;
	//private static Dimension currentRSIDim;
	int width;
	int height;
	int x0;

	boolean parametersChanged = true;
	/**
	 * 
	 */
	public MA() {
		super();
		indicatorSeries = new ArrayList();
		if (maParameters == null) {
			maParameters = new MAParameters(14);
		}
		indicatorColor = Color.yellow;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getIndicatorParameters()
	 */
	public IndicatorParameters getIndicatorParameters() {
		return maParameters;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		indicatorSeries = computeMovingAverage(maParameters.getNumberOfDays(), lDS);
		parametersChanged = true;
		//logger.debug("Init MA periode is: " + maParameters.getNumberOfDays());

	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
	 */
	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
		indicatorSeries = computeMovingAverage(maParameters.getNumberOfDays(), lDS);
		parametersChanged = true;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return true;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D, com.exponto.jstock.GraphicsParameters)
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP) {
		graphicsParameters = gP;
		logMin = graphicsParameters.getLogaritmicMinimumValue();
		logMax = graphicsParameters.getLogaritmicMaximumValue();
		int tMargin = gP.getTopMargin();
		int bMargin = gP.getBottomMargin();
		width =  gP.getNetWidth();
		height = (int) gP.getDimension().getHeight() - tMargin - bMargin;
		x0 = gP.getLeftMargin();
		int yma0 = height + tMargin;
		scaleY =  height/(logMax - logMin);
		//logger.debug("ma area - height, scaleY, yma0: " + height + ", " + scaleY + ", " + yma0);
		int widthOfDay =gP.getPixelPrDay();
		numberOfDaysToDisplay = (int) (width / widthOfDay);
		g2d.setColor(indicatorColor);
		int maDays = maParameters.getNumberOfDays();

		Double value = null;
		double ytemp, yma, yma2;
		int maLength = indicatorSeries.size() - maDays; // This is the number of valid values in maList
		int xma =  x0 + (int) widthOfDay/2;
		if (maLength < numberOfDaysToDisplay) {
			// Adjust starting point if dataseries is shorter than display
			if ((maLength + maDays) < numberOfDaysToDisplay) {
				// Also the dataseries is shorter than the display 
				xma = xma +  widthOfDay * (maDays);
				// Start at first valid value
				maLength = maDays;
			} else {
				// Only the maList is shorter than the display
				xma = xma + widthOfDay * (numberOfDaysToDisplay - maLength + 1);
				maLength = maDays;
			}
		} else {
			// Starting point within maList when valid maList is longer than display
			maLength = indicatorSeries.size() - numberOfDaysToDisplay - 1;	
		}

		Line2D maLine = new Line2D.Double();
		g2d.setColor(Color.yellow);
//		logger.debug("RsiPanel - ArraySize, Days to display: " + rsiList.size() + ", " + numberOfDaysToDisplay);
		
//		Ensure that i is within array bounds
		if (maLength < 0 ) {
			logger.debug("Set maLength to 0");
			maLength = 0;
		} else if (maLength >= indicatorSeries.size()) {
			maLength = indicatorSeries.size() -1;
		}
		// Find first valid input
		while (value == null) {
			value = (Double) indicatorSeries.get(maLength ++);
		}
		ytemp = Math.log(value.doubleValue())/Constants.log10 - logMin; 
		yma = yma0  - ytemp * scaleY ;

		for (int i = (maLength); i < indicatorSeries.size(); i++) {
			value = (Double) indicatorSeries.get(i);
			if (value != null) {
				ytemp = Math.log(value.doubleValue())/Constants.log10 - logMin;
				yma2 = yma0  - ytemp * scaleY ;
				maLine.setLine(xma, yma, xma + widthOfDay, yma2);
				g2d.draw(maLine);
				//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
				yma = yma2;
			}
			xma = xma + widthOfDay;
		}
		g2d.setColor(Color.black);
		// Update status variables
		parametersChanged = false;
	}

	public boolean hasChanged() {
	   return parametersChanged;
	}

	public String toString() {
		return "MA(" + maParameters.getNumberOfDays() + ")";
	}



	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(2);
		df.setDecimalSeparatorAlwaysShown(false);
		String value ="NA";
		int index = displayNumberToArrayIndex(n, graphicsParameters, indicatorSeries);
		if (index < indicatorSeries.size()) {
			Double  d = (Double) indicatorSeries.get(index);
			if (d != null) {
				value = df.format(d);
			}
		}
		return this.toString() + " = " + value;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int)
	 */
	public String[] printAll(int number) {
		Double d;
		String [] sA = new String[number + 1];
		int start = indicatorSeries.size() - number;
		int index = 1;
		sA[0] = "MA("+String.valueOf(maParameters.getParameters())+")";
		if (start < 0) start = 0;
		for (int i = start; i < indicatorSeries.size(); i++) {
			d = (Double) indicatorSeries.get(displayNumberToArrayIndex(i, graphicsParameters, indicatorSeries));
			sA[index++] = d.toString();
		}
		return sA;
	}

	/**
	 * Computes the moving avarages. For days without trading or faulty data
	 * null is put in the ArrayList().
	 * 
	 * @param numberOfDays how many days the Moving avarge is computed over
	 * @return an ArrayList with the moving avarages, may contain null values that 
	 * shall be ignored
	 */
	private ArrayList computeMovingAverage(int numberOfDays, LogDataSeries logDS) {
		FifoQueue maQueue = new FifoQueue(numberOfDays, new Double(0));
		ArrayList maList = new ArrayList();
		LogTradingData lTD;
		Double temp;
		double ma = 0.0;
		//logMin = graphicsParameters.getLogaritmicMinimumValue();
		//logMax = graphicsParameters.getLogaritmicMaximumValue();
		logDS.setCurrentPeriode();
		// The first numberOfDays values are not correct and thus null is added to the maList
		for (int i = 0; i < numberOfDays;) {
			lTD = logDS.getNextLTD();
			if (lTD.isTraded()) {
				temp =  (Double) maQueue.fetchAndStore(new Double(lTD.getClose()));
				ma = ma - temp.doubleValue() + lTD.getClose();
				i++;
			} 
			maList.add(null);
		}
//		Start for "real" values		
		while (logDS.hasNextLTD()) {
			lTD = logDS.getNextLTD();
			if (lTD.isTraded()) {
				temp =  (Double) maQueue.fetchAndStore(new Double(lTD.getClose()));
				ma = ma - temp.doubleValue() + lTD.getClose();
//				Computes moving avarage and stores the value
				maList.add(new Double(ma/numberOfDays));
			} else {
				// Ignores
				//logger.debug("MA ignored, null put in ArrayList");
				maList.add(null);
			}
		}
		return maList;
	}

}
