/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 05-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Contains Moving Average specific parameters used in computation of an
 * indicator series. It may also be used by related indicators using the same 
 * kind of parameters. Parameter name (for propertiies file) may be specified
 * in the Constructor.
 * 
 * @author kaare
 *
 */
public class MAParameters extends IndicatorParameters {
	
	JComponent [][] compArray = null;
	int [] numberOfDays = new int[1];
	JTextField maText;
	String propertyName;
	String toolTip = "Enter the number of days to use in Moving Average calculation. Use 0 for no MA.";
	String labelName = "MA";

	public MAParameters() {
		numberOfDays[0] = 14;
		propertyName ="MA.NumberOfDays";
	}

	
	public MAParameters(int days) {
		numberOfDays[0] = days;
		propertyName ="MA.NumberOfDays";
	}

	public MAParameters(int days, String propName, String label,String toolTipString) {
		numberOfDays[0] = days;
		propertyName =propName;
		labelName = label;
		toolTip = toolTipString;
	}


	public int [] getParameters() {
		return numberOfDays;
	}

	public void setParameters(int [] parameters) {
		numberOfDays = parameters;
	}

	
	public void setParameters(int index, int value) {
		numberOfDays[index] = value;
	}

	
	public String [] getParameterNames() {
		String [] strA = new String [1];
		strA[0] = propertyName;
		return strA;
	}


	/**
	 * @return the number of days used in the Moving Average calculation
	 */
	public int getNumberOfDays() {
		return numberOfDays[0];
	}

	/**
	 * @param i the number of days used in the Moving Average calculation
	 */
	public void setNumberOfDays(int i) {
		numberOfDays[0] = i;
	}

	public JComponent[][] getUserInterface() {
		if (compArray == null) {
			compArray = this.initUserInterface();
		}
		return compArray;
	}

	public boolean drawThis() {
		if (this.getNumberOfDays() > 0) {
			return true;
		} else return false;
	}

	public void updateParameters() {
		
		try {
			//JTextField jTF = (JTextField) cA[0][1];
			//logger.debug(" Passed maText is: " + jTF.getText());
			this.setParameters(0, Integer.parseInt(maText.getText()));            
		} catch (NumberFormatException e) {
			logger.error("Got a NumberFormatException when trying to read input for MA");
		}
		// 2 lines debug code
//		int [] dd  = this.getParameters();
//		logger.debug("Update 2: number of days is " + dd[0]);

	}

	private JComponent[][] initUserInterface() {
		JComponent [][] compArray;
	
		JLabel ma = new JLabel(labelName);
		maText = new JTextField(3);
		maText.setText(String.valueOf(this.getNumberOfDays()));
		
		maText.setToolTipText(toolTip);
		ma.setToolTipText(toolTip);

		compArray = new JComponent[1][];
		compArray[0] = new JComponent[2]; 

		compArray[0][0] = ma;
		compArray[0][1] = maText;

		return compArray;
	}


}
