/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 08-Mar-2004
 *
 */
package com.exponto.jstock.indicators;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.util.ArrayList;
import java.text.DecimalFormat;

import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.GraphicsParameters;
/**
 * @author kaare
 *
 */
public class Rsi extends Indicator {
	RsiParameters rsiParameters;
	GraphicsParameters graphicsParameters;
	protected ArrayList indicatorSeries;
	
	int oldNumberOfDays = 0;
	boolean parametersChanged = true;

//	int tMargin = 0;
//	int bMargin = 0;
	int widthOfDay;
	int numberOfDaysToDisplay;
	double scaleY = 0.01;
	//private static Dimension currentRSIDim;
	int width;
	int height;
	int x0, y0;
	
	public Rsi () {
		super();
		indicatorSeries = new ArrayList();
		rsiParameters = new RsiParameters(14);
		indicatorColor = Color.red; 
	}

	public IndicatorParameters getIndicatorParameters() {
		return rsiParameters;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(java.lang.String, com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		if (oldNumberOfDays != rsiParameters.getNumberOfDays()){
			indicatorSeries = loadRsi(lDS, rsiParameters);
		}
	}
	
	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
		if (oldNumberOfDays != rsiParameters.getNumberOfDays()){
			indicatorSeries = loadRsi(lDS, (RsiParameters) indParam);
		}
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return false;
	}
	 public boolean hasChanged() {
	 	return parametersChanged;
	 }
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D)
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP) {
		graphicsParameters = gP;
		width =  gP.getNetWidth();
		height = (int) gP.getDimension().getHeight() - gP.getTopMargin() - gP.getBottomMargin();
		x0 = gP.getLeftMargin();
		y0 = height + gP.getTopMargin();
		//y0 = tMargin;
		scaleY = (double) height/100;
		logger.debug("RsiPanel - height, scaleY, y0: " + height + ", " + scaleY + ", " + y0);
		widthOfDay =gP.getPixelPrDay();
		numberOfDaysToDisplay = (int) (width / widthOfDay);
		//g2d.setBackground(Color.lightGray);
		g2d.setColor(Color.black);
		//drawTitle(g2d);
		drawAxis(g2d);
		if (indicatorSeries == null) {
			logger.error("indicatorSeries for RSI is null");
		} else if (indicatorSeries.size() == 0){
			logger.error("indicatorSeries for RSI has size of null");
		} else {
			drawRSI(g2d);			
		}
		// Update status variables
		parametersChanged = false;
		oldNumberOfDays = rsiParameters.getNumberOfDays();
		//logger.debug("RSI panel repainted");
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		String s = " = ";
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(2);
		df.setDecimalSeparatorAlwaysShown(false);
		String value ="NA";
		int index;
		if (n < 0) {
			index = indicatorSeries.size() -1;
			s = ", last value is ";
		} else {
			index = displayNumberToArrayIndex(n, graphicsParameters, indicatorSeries);
		}
		if (index < indicatorSeries.size()) {
			Double  d = (Double) indicatorSeries.get(index);
			if (d != null) {
				value = df.format(d);
			}
		}
		return this.toString() + s + value;
	}

	public String toString() {
		return "RSI(" + rsiParameters.getNumberOfDays() + ")";
	}

//	public void printValue(int xValue){
//		String value = this.getDailyPrintOut(xValue);
//		
//	}
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int, int)
	 */
	public String[] printAll(int number) {
		Double d;
		String [] sA = new String[number + 1];
		int start = indicatorSeries.size() - number;
		int index = 1;
		sA[0] = "RSI("+String.valueOf(rsiParameters.getParameters())+")";
		if (start < 0) start = 0;
		for (int i = start; i < indicatorSeries.size(); i++) {
			d = (Double) indicatorSeries.get(displayNumberToArrayIndex(i, graphicsParameters, indicatorSeries));
			sA[index++] = d.toString();
		}
		return sA;
	}


	private void drawTitle(final Graphics2D g2d) {
		// Title
		String str = "RSI("+String.valueOf(rsiParameters.getNumberOfDays())+")";
		g2d.drawString(str, 5, (int) y0 - height);
	}
	
	private void drawAxis(final Graphics2D g2d) {
		int y;
		int limitHi = 80;
		int limitLow = 20;
		//x-axis
		Line2D line = new Line2D.Double (x0, y0, x0 + width, y0);
		g2d.draw(line);
		//y-axis
		line.setLine(x0, y0, x0, y0 - height);
		g2d.draw(line);
		// Lower limit line & text
		g2d.setColor(Color.gray);
		y = y0  - (int) (limitLow * scaleY) ;
		line.setLine(x0, y, x0+ width, y);
		g2d.draw(line);
		String str = String.valueOf(limitLow);
		g2d.drawString(str, 15, (int) y+5);
//		High limit line & text
		y = y0  - (int) (limitHi * scaleY) ;
		line.setLine(x0, y, x0+ width, y);
		g2d.draw(line);
		str = String.valueOf(limitHi);
		g2d.drawString(str, 15, (int) y+5);
	}

	private void drawRSI(final Graphics2D g2d) {
		Double value;
		double y, y2;
		int length = indicatorSeries.size();
		int x = x0 - (int) widthOfDay/2;
		if (length < numberOfDaysToDisplay) {
			// Adjust starting point if dataseries is shorter than display
			if ((length + rsiParameters.getNumberOfDays()) < numberOfDaysToDisplay) {
				x = x +  widthOfDay * (rsiParameters.getNumberOfDays());
			} else {
				x = x + widthOfDay * (numberOfDaysToDisplay - length);
			}
		}
		Line2D line = new Line2D.Double ();
		g2d.setColor(indicatorColor);
//		logger.debug("RsiPanel - ArraySize, Days to display: " + rsiList.size() + ", " + numberOfDaysToDisplay);
		length = length - numberOfDaysToDisplay - 1;
//		Ensure that i is within array bounds
		if (length < 0) {
//			x = x - widthOfDay*(index);
			length = 0;
		}
		value = (Double) indicatorSeries.get(length ++);
		y = y0  - value.doubleValue() * scaleY ;
		//for (int i = (rsiList.size() - numberOfDaysToDisplay -1); i < rsiList.size(); i++) {
		for (int i = (length); i < indicatorSeries.size(); i++) {
			// Ensure that i is within array bounds
			if (i < 0) i = 0;
			value = (Double) indicatorSeries.get(i);
			//y = y0  - value.doubleValue() * scaleY ;
			y2 = y0  - value.doubleValue() * scaleY ;
			//line.setLine(x, y, x + widthOfDay, y);
			line.setLine(x, y, x + widthOfDay, y2);
			x = x + widthOfDay;
			g2d.draw(line);
			//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
			y = y2;
		}
		//logger.debug("RsiPanel - x, y, value: " + x + ", "+ y + ", " + value);
	}

	private ArrayList loadRsi(LogDataSeries lDS, RsiParameters rsiParam) {
		parametersChanged = true;
		return RsiAlgorithm.findRSIClassicWay(lDS, rsiParam);
	}

	
}
