/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * jStock.java
 * 
 * Whats next:	1) add special events (ex-dividend, splitt ...)
 *	 					2) Internationalization
 *						3) Redesign time again???  
 *  				
 *
 */

package com.exponto.jstock;
/**
 * The "main" class, generates the user interface, but tries as hard as it 
 * can to let other classes do the work. 
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.ArrayList;
import org.apache.log4j.Logger;
//import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.PropertyConfigurator;

import com.exponto.jstock.popupwindows.*;

//import java.io.IOException;
import java.net.URL;


public class jStock extends JFrame {
    private GraphicsConfiguration gc;
    private Rectangle bounds;
    private JTabbedPane tabbedPane;
    private JPanel controlPanel;
	private ImageIcon weeklyIcon;
	private static Notes notes;
	
	// Log4j
	static Logger logger = Logger.getLogger(jStock.class);
	
//	Keep track of currently selected tab    
    private int tabIndex = 0;
	private HashMap tabMap = new HashMap();
    
    // Userpreference is common for all panels 
    private static UserPreferences userPref;
 
    // For controlpanel
	private JCheckBox drawBox;
    private JCheckBox weeklyButton;
	private JCheckBox notesBox;
    private JLabel ticker;
    private JTextField tickerInput;
    private JButton getStock;
    private JButton options;
    private JButton indicators;
    private JButton closeWin;
    private JButton quit;
//	private JButton about;
	
	// Size params
	private static Dimension tabbedDim;
	int lostWidth = 6;	// 8?   Lost to borders etc.
	int lostHeight = 67; // 61,49? Lost to borders & controlPanel etc.   
   
    /** Creates new jStock */
    public jStock() {
        super("jStock charts");
        gc = this.getGraphicsConfiguration();
	//Gets (virtual) screensize
        bounds = gc.getBounds();
        // Use with x & y offset in userpreferences
        this.setLocation(bounds.x, bounds.y + 50);
//        System.out.println(bounds.toString());
        userPref = UserPreferences.getInstance();
        initComponents();
        ChartPanel.setAvailableSize(tabbedDim);
		if (userPref.getEnableDB()) {
			loadWatchList();	
		}
    }

    private void initComponents() {
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
				close();
            }
			public void windowStateChanged(WindowEvent evt) {
				changeSize();
				logger.debug("Window state changed (will resize)");		
			}
        });
		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				changeSize();
				logger.debug(" jStock resized, new tabbedSize is: " + tabbedDim.toString());
			}
		 });
		
		weeklyIcon = new ImageIcon("weekly-s.png");
        tabbedPane = new JTabbedPane();
        // Scroll tabs if all tabs don't fit in one line.
        // Mostly added to get control over the available height of the 
        //JTabbedPane. It is possible to use getTabRunCount() if exact 
        //height of the tabs are known.
        tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
        // Set initial size of Panel
        tabbedDim = userPref.getUserDim();
        tabbedPane.setPreferredSize(tabbedDim);
		tabbedPane.addChangeListener(new jStockTabbedListener()); 
        addControlPanel();
//		Draws area for tabbedpane even if empty
        getContentPane().add(tabbedPane, BorderLayout.CENTER);
        getContentPane().add(controlPanel, BorderLayout.SOUTH);
//		Is not as easy as this
		controlPanel.requestFocusInWindow();        
		tickerInput.requestFocusInWindow();
        pack();
        setVisible(true);
    }

	private void loadWatchList() {
		JstockDB jDB = JstockDB.getInstance();
		ArrayList watchList = jDB.loadWatchList();
		TickerID watch;
		for (int i =0; i < watchList.size(); i++) {
			watch = (TickerID) watchList.get(i);
			getStock(watch.getTickerName());
		}
	}
	
	private void changeSize() {
		tabbedDim.setSize(this.getWidth() - lostWidth, this.getHeight() - lostHeight);
		ChartPanel.setAvailableSize(tabbedDim);
	//logger.info(" Size of tabbedpane in paint is " + tabbedDim.toString());
	}
    private void addControlPanel() {
        controlPanel = new JPanel();
        controlPanel.setLayout(new FlowLayout());
		drawBox = new JCheckBox("Draw");
		drawBox.setMnemonic(KeyEvent.VK_D); 
		drawBox.setSelected(userPref.getDrawMode());
		weeklyButton = new JCheckBox("Show weekly");
		weeklyButton.setMnemonic(KeyEvent.VK_W); 
		weeklyButton.setSelected(userPref.getShowWeekly());
		notesBox = new JCheckBox("Notes");
		notesBox.setMnemonic(KeyEvent.VK_N);
		notesBox.setSelected(userPref.getNotes());
        ticker = new JLabel("Ticker");
        tickerInput = new JTextField(5);
        //tickerInput.createDefaultModel();
        getStock = new JButton("Get");
		getStock.setMnemonic(KeyEvent.VK_G);
        options = new JButton("Options");
		options.setMnemonic(KeyEvent.VK_O);
        indicators = new JButton("Indicators");
		indicators.setMnemonic(KeyEvent.VK_I);
        closeWin = new JButton("Close");
		closeWin.setMnemonic(KeyEvent.VK_C);
        quit = new JButton("Quit");
		quit.setMnemonic(KeyEvent.VK_Q);
//		about = new JButton("About");
//		about.setMnemonic(KeyEvent.VK_A);

		controlPanel.add(drawBox);
		controlPanel.add(weeklyButton);
		if (userPref.getEnableDB()) {
// TODO: Need to redraw controlPanel if db support is enabled in options			
			controlPanel.add(notesBox);			
		}
		
        controlPanel.add(ticker);
        controlPanel.add(tickerInput);
        controlPanel.add(getStock);
		controlPanel.add(options);
        //controlPanel.add(indicators);
        controlPanel.add(closeWin);
        controlPanel.add(quit);
//		controlPanel.add(about);
		
		drawBox.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				changeDrawBoxState();
			}
		});		
		weeklyButton.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				changeWeeklyBoxState();
			}
		});
		notesBox.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				changeNotesState();
			}
		}); 
 
        tickerInput.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
				String s = tickerInput.getText();
				tickerInput.setText(null);
				getStock(s);
            }
        }); 
        getStock.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
				String s = tickerInput.getText();
				tickerInput.setText(null);
                getStock(s);
            }
        });
        options.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
                showOptionsMenue();
            }
        });
//        indicators.addActionListener(new ActionListener () {
//            public void actionPerformed(ActionEvent e) {
//                showIndicatorsMenue();
//            }
//        });
        closeWin.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
                closeWin();
            }
        });
        quit.addActionListener(new ActionListener () {
            public void actionPerformed(ActionEvent e) {
				close();
            }
        });
//		about.addActionListener(new ActionListener () {
//			public void actionPerformed(ActionEvent e) {
//				about();
//			}
//		});
    }
    
	private void changeDrawBoxState() {
		userPref.setDrawMode(!userPref.getDrawMode()); 
//		System.out.println(" Trykker p knappen og n er den " + userPref.getShowWeekly());  	
	}
    private void changeWeeklyBoxState() {
		userPref.setShowWeekly(!userPref.getShowWeekly()); 
//		System.out.println(" Trykker p knappen og n er den " + userPref.getShowWeekly());  	
    }
	private void changeNotesState() {
		userPref.setNotes(!userPref.getNotes());
		if (userPref.getNotes()) {
			notes = Notes.getInstance();
//			notes = Notes.getInstance(ticker, userPref.getExchange());
		}
	}

    private void getStock(String s){
//  		Only adds if it dosn't exists already        
      	if (!tabMap.containsKey(s)) {
      		tabIndex = tabMap.size();
      		Integer value = new Integer(tabIndex);
			ChartPanel cP = null;
      		try {
				cP = new ChartPanel(s);
      		} catch (TickerNotFoundException e) {
      			logger.error(e.getMessage());
				return;     			
      		}
      		if (cP != null) {
				tabMap.put(s, value);
				tabbedPane.addTab(s, null, cP, null);
				if (userPref.getShowWeekly()) {
					//cP.getChartData();
					ChartPanel cPW = new ChartPanel(cP.getChartData());
					tabbedPane.addTab(s + " ", weeklyIcon , cPW , null);
					tabIndex = tabMap.size();
					value = new Integer(tabIndex);
					//TODO: How to keep track of weekly and daily chart if " " is not used? See close()
					tabMap.put(s + " ", value);
				}
      		} else {
      			logger.error(" Loading chart for ticker " + s + " failed");
      		}
      	} else {
      		Integer tab = (Integer) tabMap.get(s);
      		tabIndex = tab.intValue();
      	}
      	tabbedPane.setSelectedIndex(tabIndex);
        pack();
        setVisible(true);
//		if (userPref.getNotes()) {
//			if (notes == null) {
//				notes = Notes.getInstance(s, userPref.getExchange());
//			} else {
//				notes.refresh(s, userPref.getExchange());
//			}
//		}
    }

    private void showOptionsMenue(){
    	PreferenceEditor prefEd = new PreferenceEditor();
    }
//    private void showIndicatorsMenue(){
//    	// will probably never be used
//        System.out.println("Her kommer indiktor menyen");
//        System.out.println("    -volum?");
//        System.out.println("    -rsi");
//        System.out.println("    -historisk volatilitet");
//    }
    private void closeWin(){
    	tabIndex = tabbedPane.getSelectedIndex();
    	String s = tabbedPane.getTitleAt(tabIndex);
    	tabMap.remove(s);
		tabbedPane.remove(tabIndex);
    }
    
//    private void about() {
//    	About about = new About();
//    }
 
 	private void close() {
		userPref.setXSize(this.getWidth() - lostWidth); // + 5
		userPref.setYSize(this.getHeight() - lostHeight); // + 27
		logger.debug("Saving current window size: "+ this.getWidth() + ", "+ this.getHeight());
//		System.out.println("Saving current window size: "+ this.getWidth() + ", "+ this.getHeight());
		userPref.saveAll();
		// Close DB connection if it exists
		DBManager justClosing = DBManager.getInstance();
		justClosing.closeConnection();
		logger.info(" Saved preferences and closed existing DB connections");
//		  System.out.println("Saving preferences");
		System.exit(0); 		  
 	}

//	public static void refreshControlPanel() {
//		addControlPanel();
//	}
	public void paint(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		super.paint(g2d);
	}
        
    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
		String resource ="/log.properties";
		//System.out.println("pwd: " + System.getProperty("user.dir"));
		//System.out.println("cp: " + System.getProperty("java.class.path"));
		URL configFileResource = jStock.class.getResource(resource);
		//System.out.println("cp: " + configFileResource.toString());
		PropertyConfigurator.configure(configFileResource);    	
		logger.info(" Loading application");
        jStock taa = new jStock();
        //taa.setVisible(true);
    }
}

