/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 23.des.2003
 *
 */
package com.exponto.util;

/**
 * A very simple First In First Out (FIFO) queue for storing a
 * number of objects. 
 * 
 * The size of the list is fixed when the
 * instance of the class is created.
 * 
 * "null" objects are allowed, but the queue maybe created with an
 * initial value.
 * 
 * @author ttranu
 *
 */
public class FifoQueue {
	Object[] data;
	int next = 0;
	int maxCount;
	public FifoQueue(int size) {
		data = new Object[size];
		maxCount = size -1;
	}
	public FifoQueue(int size, Object initialValue) {
		data = new Object[size];
		maxCount = size -1;
		for (int i = 0; i <= maxCount; i++) {
			data[i] = initialValue;
		}
	}

	public Object fetchAndStore(Object obj) {
		Object out = data[next];
		data[next++] = obj;
		checkNext();
//		System.out.println("In is: " + obj + " Out is: " + out);
		return out;
	}
	private void checkNext() {
		if (next > maxCount) {
			next = 0;
		}
	}
}
